package com.gic.auth.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.AccountGroupMemberTypeEnum;
import com.gic.auth.constant.SignConstants;
import com.gic.auth.dto.AccountGroupDTO;
import com.gic.auth.dto.AccountGroupListDTO;
import com.gic.auth.dto.AccountGroupMemberDTO;
import com.gic.auth.dto.AccountGroupRelDTO;
import com.gic.auth.entity.TabSysAccountGroup;
import com.gic.auth.entity.TabSysAccountGroupRel;
import com.gic.auth.entity.TabSysCollaborator;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
/**
 * 
 * @ClassName: AccountGroupApiServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 9:36
 */
@Service("accountGroupApiService")
public class AccountGroupApiServiceImpl implements AccountGroupApiService {
    @Autowired
    private AccountGroupService accountGroupService;
    @Autowired
    private AccountGroupRelService accountGroupRelService;
    @Autowired
    private UserService userService;
    @Autowired
    private CollaboratorService collaboratorService;

    @Override
    public ServiceResponse<Integer> save(AccountGroupDTO dto) {
        //参数校验
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(dto, AccountGroupDTO.SaveValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        if (accountGroupService.isAccountGroupNameRepeat(dto.getEnterpriseId(), null, dto.getAccountGroupName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组名称不能重复");
        }
        //sort值设置
        int maxSortValue = accountGroupService.getMaxSortValueByEnterpriseId(dto.getEnterpriseId());
        dto.setSort(maxSortValue + 1);
        return ServiceResponse.success(accountGroupService.save(dto));
    }

    @Override
    public ServiceResponse<Integer> update(AccountGroupDTO dto) {
        TabSysAccountGroup record = accountGroupService.getById(dto.getAccountGroupId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组id有误，查无数据");
        }
        //参数校验
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(dto, AccountGroupDTO.EditValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        if (accountGroupService.isAccountGroupNameRepeat(dto.getEnterpriseId(), dto.getAccountGroupId(),
                dto.getAccountGroupName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组名称不能重复");
        }
        dto.setEnterpriseId(record.getEnterpriseId());
        dto.setSort(record.getSort());

        return ServiceResponse.success(accountGroupService.update(dto));
    }

    @Override
    public ServiceResponse<List<AccountGroupDTO>> listAccountGroupByEnterpriseId(Integer enterpriseId) {
        List<TabSysAccountGroup> list = accountGroupService.listAccountGroupByEnterpriseId(enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            //求分组下的组员数量值
            Map<String, Integer> map = new HashMap<>(16);
            List<Integer> accountGroupIdList = new ArrayList<>(list.size());
            for (TabSysAccountGroup temp : list) {
                accountGroupIdList.add(temp.getAccountGroupId());
                //设置默认值0
                map.put(temp.getAccountGroupId().toString(), 0);
            }
            List<AccountGroupListDTO> accountGroupListDTOList = accountGroupRelService
                    .groupByAccountGroupId(accountGroupIdList);

            if (CollectionUtils.isNotEmpty(accountGroupListDTOList)) {
                for (AccountGroupListDTO dto : accountGroupListDTOList) {
                    map.put(dto.getAccountGroupId().toString(), dto.getMemberCount());
                }
            }
            List<AccountGroupDTO> result = EntityUtil.changeEntityListNew(AccountGroupDTO.class, list);
            for (AccountGroupDTO temp : result) {
                temp.setMemberCount(map.get(temp.getAccountGroupId().toString()));
            }
            return ServiceResponse.success(result);
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    @Override
    public ServiceResponse<List<AccountGroupDTO>> listNoMemberByEnterpriseId(Integer enterpriseId) {
        List<TabSysAccountGroup> list = accountGroupService.listAccountGroupByEnterpriseId(enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(AccountGroupDTO.class, list));
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<AccountGroupDTO> deleteByAccountGroupId(Integer accountGroupId) {
        TabSysAccountGroup record = accountGroupService.getById(accountGroupId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组id有误，查无数据");
        }
        //逻辑删除分组
        accountGroupService.updateStatusByAccountGroupId(accountGroupId);
        //删除组员分组关系数据
        accountGroupRelService.deleteByAccountGroupId(accountGroupId);
        return ServiceResponse.success(EntityUtil.changeEntityNew(AccountGroupDTO.class, record));
    }

    @Override
    public ServiceResponse<AccountGroupDTO> sort(Integer accountGroupId, Integer sort) {
        TabSysAccountGroup record = accountGroupService.getById(accountGroupId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组id有误，查无数据");
        }
        accountGroupService.setSort(record, sort);
        return ServiceResponse.success(EntityUtil.changeEntityNew(AccountGroupDTO.class, record));
    }

    @Override
    public ServiceResponse<List<AccountGroupMemberDTO>> listAccountGroupMemberByAccountGroupId(List<Integer> accountGroupIdList) {
        List<AccountGroupMemberDTO> result = new ArrayList<>();
        List<TabSysAccountGroupRel> list = accountGroupRelService.listByAccountGroupId(accountGroupIdList, null);
        //查询管理员列表
        result.addAll(getMemberOfUser(list, true, null, null));
        // 查询协作人列表
        result.addAll(getMemberOfCollaborator(list, true, null, null));
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<AccountGroupMemberDTO>> listNotInAccountGroupByAccountGroupId(Integer accountGroupId,
            String search) {
        TabSysAccountGroup record = accountGroupService.getById(accountGroupId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组id有误，查无数据");
        }
        List<AccountGroupMemberDTO> result = new ArrayList<>();
        List<TabSysAccountGroupRel> list = listByAccountGroupId(accountGroupId);
        //查询管理员列表
        result.addAll(getMemberOfUser(list, false, search, record.getEnterpriseId()));
        // 查询协作人列表
        result.addAll(getMemberOfCollaborator(list, false, search, record.getEnterpriseId()));
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<AccountGroupMemberDTO>> listAllMember(String search, Integer enterpriseId) {
        List<AccountGroupMemberDTO> result = new ArrayList<>();
        //查询管理员列表
        result.addAll(getMemberOfUser(null, false, search, enterpriseId));
        // 查询协作人列表
        result.addAll(getMemberOfCollaborator(null, false, search, enterpriseId));
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<AccountGroupMemberDTO>> listInAccountGroupByAccountGroupId(Integer accountGroupId,
            String search) {
        TabSysAccountGroup record = accountGroupService.getById(accountGroupId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组id有误，查无数据");
        }
        List<AccountGroupMemberDTO> result = new ArrayList<>();
        List<TabSysAccountGroupRel> list = listByAccountGroupId(accountGroupId);
        //查询管理员列表
        result.addAll(getMemberOfUser(list, true, search, record.getEnterpriseId()));
        // 查询协作人列表
        result.addAll(getMemberOfCollaborator(list, true, search, record.getEnterpriseId()));

        return ServiceResponse.success(result);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<String> saveAccountGroupRelFetch(List<AccountGroupMemberDTO> list) {
        if (CollectionUtils.isNotEmpty(list)) {
            /*
            //逻辑删除之前的数据，先删后增
            //暂时不删除，因为是批量新增部分，而不是全部，所以不需要删除之前的数据
             */
            List<AccountGroupRelDTO> result = new ArrayList<>(list.size());
            StringBuilder operationObject = new StringBuilder();
            Date now = new Date();
            for (AccountGroupMemberDTO dto : list) {
                AccountGroupRelDTO temp = new AccountGroupRelDTO();
                temp.setStatus(1);
                if (dto.getAccountGroupId() == null) {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分组ID不能改为空");
                }
                if (dto.getUserId() == null) {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "组员ID不能改为空");
                }
                temp.setAccountGroupId(dto.getAccountGroupId());
                temp.setCreateTime(now);
                temp.setUpdateTime(now);
                temp.setUserId(dto.getUserId());
                temp.setType(dto.getType());
                temp.setEnterpriseId(dto.getEnterpriseId());
                result.add(temp);

                operationObject.append(dto.getMemberName()).append(SignConstants.COMMA);
            }
            accountGroupRelService.insertFetch(result);

            TabSysAccountGroup accountGroup = accountGroupService.getById(list.get(0).getAccountGroupId());
            if (accountGroup != null) {
                operationObject.append("-分组到-").append(accountGroup.getAccountGroupName());
            }

            return ServiceResponse.success(operationObject.toString());
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "无数据");
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<String> groupingOfUser(List<Integer> userIdList, List<Integer> accountGroupId,
            Integer enterpriseId) {
        return grouping(userIdList, accountGroupId, enterpriseId, AccountGroupMemberTypeEnum.ADMIN.getCode());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<String> groupingOfCollaborator(List<Integer> collaboratorIdList, List<Integer> accountGroupId,
            Integer enterpriseId) {
        return grouping(collaboratorIdList, accountGroupId, enterpriseId,
                AccountGroupMemberTypeEnum.COLLABORATOR.getCode());
    }

    /**
     * 查询分组下的组员列表数据
     * @param accountGroupId
     * @return
     */
    private List<TabSysAccountGroupRel> listByAccountGroupId(Integer accountGroupId) {
        return accountGroupRelService.listByAccountGroupId(accountGroupId);
    }

    private List<Integer> getUserIdList(int type, List<TabSysAccountGroupRel> accountGroupRelList) {
        List<Integer> userIdList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(accountGroupRelList)) {
            for (TabSysAccountGroupRel rel : accountGroupRelList) {
                if (rel.getType().intValue() == type) {
                    userIdList.add(rel.getUserId());
                } else if (rel.getType().intValue() == type) {
                    userIdList.add(rel.getUserId());
                }
            }
        }
        return userIdList;
    }

    /**
     * 获取管理员列表
     * @param accountGroupRelList
     * @param isIn 是否在分组内
     * @return
     */
    private List<AccountGroupMemberDTO> getMemberOfUser(List<TabSysAccountGroupRel> accountGroupRelList, boolean isIn,
            String search, Integer enterpriseId) {
        List<Integer> userIdList = getUserIdList(AccountGroupMemberTypeEnum.ADMIN.getCode(), accountGroupRelList);
        List<TabSysUser> userList;
        //查询管理员列表
        if (isIn) {
            userList = userService.listUserByIdList(userIdList);
        } else {
            userList = userService.listUserNotInIdList(userIdList, search, enterpriseId);
        }
        return userToMember(userList);
    }

    private List<AccountGroupMemberDTO> userToMember(List<TabSysUser> userList) {
        List<AccountGroupMemberDTO> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(userList)) {
            for (TabSysUser user : userList) {
                AccountGroupMemberDTO dto = new AccountGroupMemberDTO();
                dto.setMemberName(user.getUserName());
                dto.setUserId(user.getUserId());
                dto.setPhone(user.getPhoneNumber());
                dto.setType(AccountGroupMemberTypeEnum.ADMIN.getCode());
                result.add(dto);
            }
        }
        return result;
    }

    private List<AccountGroupMemberDTO> getMemberOfCollaborator(List<TabSysAccountGroupRel> accountGroupRelList,
            boolean isIn, String search, Integer enterpriseId) {
        List<Integer> collaboratorIdList = getUserIdList(AccountGroupMemberTypeEnum.COLLABORATOR.getCode(),
                accountGroupRelList);
        List<TabSysCollaborator> collaboratorList;
        if (isIn) {
            collaboratorList = collaboratorService.listByCollaboratorIdList(collaboratorIdList);
        } else {
            collaboratorList = collaboratorService.listNotInCollaboratorIdList(collaboratorIdList, search, enterpriseId);
        }
        return collaboratorToMember(collaboratorList);
    }

    private List<AccountGroupMemberDTO> collaboratorToMember(List<TabSysCollaborator> userList) {
        List<AccountGroupMemberDTO> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(userList)) {
            for (TabSysCollaborator user : userList) {
                AccountGroupMemberDTO dto = new AccountGroupMemberDTO();
                dto.setMemberName(user.getCollaboratorName());
                dto.setUserId(user.getCollaboratorId());
                dto.setPhone(user.getPhone());
                dto.setType(AccountGroupMemberTypeEnum.COLLABORATOR.getCode());
                result.add(dto);
            }
        }
        return result;
    }

    private ServiceResponse<String> grouping(List<Integer> userIdList, List<Integer> accountGroupId, Integer enterpriseId,
            int type) {
        //先删除
        accountGroupRelService.deleteByUserId(userIdList, type);
        Date now = new Date();
        for (Integer groupId : accountGroupId) {
            List<AccountGroupRelDTO> result = new ArrayList<>();
            for (Integer userId : userIdList) {
                AccountGroupRelDTO temp = new AccountGroupRelDTO();
                temp.setStatus(1);
                temp.setAccountGroupId(groupId);
                temp.setCreateTime(now);
                temp.setUpdateTime(now);
                temp.setUserId(userId);
                temp.setType(type);
                temp.setEnterpriseId(enterpriseId);
                result.add(temp);
            }
            accountGroupRelService.insertFetch(result);
        }
        //获取日志操作对象内容
        List<TabSysUser> userList = userService.listUserByIdList(userIdList);
        StringBuilder sb = new StringBuilder();
        if (CollectionUtils.isNotEmpty(userIdList)) {
            for (TabSysUser user : userList) {
                sb.append(user.getUserName()).append(SignConstants.COMMA);
            }
            sb.append("-分组到-");
            List<TabSysAccountGroup> groupList = accountGroupService.listByIdList(accountGroupId);
            if (CollectionUtils.isNotEmpty(groupList)) {
                for (TabSysAccountGroup group : groupList) {
                    sb.append(group.getAccountGroupName()).append(SignConstants.COMMA);
                }
            }
        }
        return ServiceResponse.success(sb.toString());
    }
}
